from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.base_incoming_feature_feature_type import BaseIncomingFeatureFeatureType
from ..models.integration_type import IntegrationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_incoming_feature_extra_properties import BaseIncomingFeatureExtraProperties
    from ..models.recipient import Recipient
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="RingCentralEmailIntegration")


@_attrs_define
class RingCentralEmailIntegration:
    """
    Attributes:
        type (IntegrationType): Type of the integration. (For instance, "API" for API Integration)
        name (str): Name of the integration. Name must be unique for each integration
        id (Union[Unset, str]):
        enabled (Union[Unset, bool]): This parameter is for specifying whether the integration will be enabled or not
        owner_team (Union[Unset, TeamMeta]):
        is_global (Union[Unset, bool]):
        field_read_only (Union[Unset, List[str]]):
        suppress_notifications (Union[Unset, bool]): If enabled, notifications that come from alerts will be suppressed.
            Defaults to false
        ignore_teams_from_payload (Union[Unset, bool]): If enabled, the integration will ignore teams sent in request
            payloads. Defaults to false
        ignore_recipients_from_payload (Union[Unset, bool]): If enabled, the integration will ignore recipients sent in
            request payloads. Defaults to false
        recipients (Union[Unset, List['Recipient']]): Optional user, schedule, teams or escalation names to calculate
            which users will receive the notifications of the alert. Recipients which are exceeding the limit are ignored
        is_advanced (Union[Unset, bool]):
        ignore_responders_from_payload (Union[Unset, bool]):
        ignore_tags_from_payload (Union[Unset, bool]):
        ignore_extra_properties_from_payload (Union[Unset, bool]):
        responders (Union[Unset, List['Recipient']]):
        priority (Union[Unset, str]):
        custom_priority (Union[Unset, str]):
        tags (Union[Unset, List[str]]):
        extra_properties (Union[Unset, BaseIncomingFeatureExtraProperties]):
        assigned_team (Union[Unset, TeamMeta]):
        feature_type (Union[Unset, BaseIncomingFeatureFeatureType]):
        email_address (Union[Unset, str]):
        email_username (Union[Unset, str]): The username part of the email address. It must be unique for each
            integration
    """

    type: IntegrationType
    name: str
    id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    is_global: Union[Unset, bool] = UNSET
    field_read_only: Union[Unset, List[str]] = UNSET
    suppress_notifications: Union[Unset, bool] = UNSET
    ignore_teams_from_payload: Union[Unset, bool] = UNSET
    ignore_recipients_from_payload: Union[Unset, bool] = UNSET
    recipients: Union[Unset, List["Recipient"]] = UNSET
    is_advanced: Union[Unset, bool] = UNSET
    ignore_responders_from_payload: Union[Unset, bool] = UNSET
    ignore_tags_from_payload: Union[Unset, bool] = UNSET
    ignore_extra_properties_from_payload: Union[Unset, bool] = UNSET
    responders: Union[Unset, List["Recipient"]] = UNSET
    priority: Union[Unset, str] = UNSET
    custom_priority: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    extra_properties: Union[Unset, "BaseIncomingFeatureExtraProperties"] = UNSET
    assigned_team: Union[Unset, "TeamMeta"] = UNSET
    feature_type: Union[Unset, BaseIncomingFeatureFeatureType] = UNSET
    email_address: Union[Unset, str] = UNSET
    email_username: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        name = self.name

        id = self.id

        enabled = self.enabled

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        is_global = self.is_global

        field_read_only: Union[Unset, List[str]] = UNSET
        if not isinstance(self.field_read_only, Unset):
            field_read_only = self.field_read_only

        suppress_notifications = self.suppress_notifications

        ignore_teams_from_payload = self.ignore_teams_from_payload

        ignore_recipients_from_payload = self.ignore_recipients_from_payload

        recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.recipients, Unset):
            recipients = []
            for recipients_item_data in self.recipients:
                recipients_item = recipients_item_data.to_dict()
                recipients.append(recipients_item)

        is_advanced = self.is_advanced

        ignore_responders_from_payload = self.ignore_responders_from_payload

        ignore_tags_from_payload = self.ignore_tags_from_payload

        ignore_extra_properties_from_payload = self.ignore_extra_properties_from_payload

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        priority = self.priority

        custom_priority = self.custom_priority

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        extra_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extra_properties, Unset):
            extra_properties = self.extra_properties.to_dict()

        assigned_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assigned_team, Unset):
            assigned_team = self.assigned_team.to_dict()

        feature_type: Union[Unset, str] = UNSET
        if not isinstance(self.feature_type, Unset):
            feature_type = self.feature_type.value

        email_address = self.email_address

        email_username = self.email_username

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if is_global is not UNSET:
            field_dict["isGlobal"] = is_global
        if field_read_only is not UNSET:
            field_dict["_readOnly"] = field_read_only
        if suppress_notifications is not UNSET:
            field_dict["suppressNotifications"] = suppress_notifications
        if ignore_teams_from_payload is not UNSET:
            field_dict["ignoreTeamsFromPayload"] = ignore_teams_from_payload
        if ignore_recipients_from_payload is not UNSET:
            field_dict["ignoreRecipientsFromPayload"] = ignore_recipients_from_payload
        if recipients is not UNSET:
            field_dict["recipients"] = recipients
        if is_advanced is not UNSET:
            field_dict["isAdvanced"] = is_advanced
        if ignore_responders_from_payload is not UNSET:
            field_dict["ignoreRespondersFromPayload"] = ignore_responders_from_payload
        if ignore_tags_from_payload is not UNSET:
            field_dict["ignoreTagsFromPayload"] = ignore_tags_from_payload
        if ignore_extra_properties_from_payload is not UNSET:
            field_dict["ignoreExtraPropertiesFromPayload"] = ignore_extra_properties_from_payload
        if responders is not UNSET:
            field_dict["responders"] = responders
        if priority is not UNSET:
            field_dict["priority"] = priority
        if custom_priority is not UNSET:
            field_dict["customPriority"] = custom_priority
        if tags is not UNSET:
            field_dict["tags"] = tags
        if extra_properties is not UNSET:
            field_dict["extraProperties"] = extra_properties
        if assigned_team is not UNSET:
            field_dict["assignedTeam"] = assigned_team
        if feature_type is not UNSET:
            field_dict["feature-type"] = feature_type
        if email_address is not UNSET:
            field_dict["emailAddress"] = email_address
        if email_username is not UNSET:
            field_dict["emailUsername"] = email_username

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_incoming_feature_extra_properties import BaseIncomingFeatureExtraProperties
        from ..models.recipient import Recipient
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        type = IntegrationType(d.pop("type"))

        name = d.pop("name")

        id = d.pop("id", UNSET)

        enabled = d.pop("enabled", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        is_global = d.pop("isGlobal", UNSET)

        field_read_only = cast(List[str], d.pop("_readOnly", UNSET))

        suppress_notifications = d.pop("suppressNotifications", UNSET)

        ignore_teams_from_payload = d.pop("ignoreTeamsFromPayload", UNSET)

        ignore_recipients_from_payload = d.pop("ignoreRecipientsFromPayload", UNSET)

        recipients = []
        _recipients = d.pop("recipients", UNSET)
        for recipients_item_data in _recipients or []:
            recipients_item = Recipient.from_dict(recipients_item_data)

            recipients.append(recipients_item)

        is_advanced = d.pop("isAdvanced", UNSET)

        ignore_responders_from_payload = d.pop("ignoreRespondersFromPayload", UNSET)

        ignore_tags_from_payload = d.pop("ignoreTagsFromPayload", UNSET)

        ignore_extra_properties_from_payload = d.pop("ignoreExtraPropertiesFromPayload", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Recipient.from_dict(responders_item_data)

            responders.append(responders_item)

        priority = d.pop("priority", UNSET)

        custom_priority = d.pop("customPriority", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        _extra_properties = d.pop("extraProperties", UNSET)
        extra_properties: Union[Unset, BaseIncomingFeatureExtraProperties]
        if isinstance(_extra_properties, Unset):
            extra_properties = UNSET
        else:
            extra_properties = BaseIncomingFeatureExtraProperties.from_dict(_extra_properties)

        _assigned_team = d.pop("assignedTeam", UNSET)
        assigned_team: Union[Unset, TeamMeta]
        if isinstance(_assigned_team, Unset):
            assigned_team = UNSET
        else:
            assigned_team = TeamMeta.from_dict(_assigned_team)

        _feature_type = d.pop("feature-type", UNSET)
        feature_type: Union[Unset, BaseIncomingFeatureFeatureType]
        if isinstance(_feature_type, Unset):
            feature_type = UNSET
        else:
            feature_type = BaseIncomingFeatureFeatureType(_feature_type)

        email_address = d.pop("emailAddress", UNSET)

        email_username = d.pop("emailUsername", UNSET)

        ring_central_email_integration = cls(
            type=type,
            name=name,
            id=id,
            enabled=enabled,
            owner_team=owner_team,
            is_global=is_global,
            field_read_only=field_read_only,
            suppress_notifications=suppress_notifications,
            ignore_teams_from_payload=ignore_teams_from_payload,
            ignore_recipients_from_payload=ignore_recipients_from_payload,
            recipients=recipients,
            is_advanced=is_advanced,
            ignore_responders_from_payload=ignore_responders_from_payload,
            ignore_tags_from_payload=ignore_tags_from_payload,
            ignore_extra_properties_from_payload=ignore_extra_properties_from_payload,
            responders=responders,
            priority=priority,
            custom_priority=custom_priority,
            tags=tags,
            extra_properties=extra_properties,
            assigned_team=assigned_team,
            feature_type=feature_type,
            email_address=email_address,
            email_username=email_username,
        )

        ring_central_email_integration.additional_properties = d
        return ring_central_email_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
    Global constants definition file
"""

import importlib.metadata

VERSION = importlib.metadata.version("tygenie")
ADD_NOTE_SCREEN_NAME = "add_note"
SETTINGS_SCREEN_NAME = "settings"
ALERTS_SCREEN_NAME = "alerts"



from enum import Enum


class CallbackConditionField(str, Enum):
    ACTIONS = "actions"
    ALIAS = "alias"
    DESCRIPTION = "description"
    ENTITY = "entity"
    EXTRA_PROPERTIES = "extra-properties"
    MESSAGE = "message"
    PRIORITY = "priority"
    RECIPIENTS = "recipients"
    SOURCE = "source"
    TAGS = "tags"
    TEAMS = "teams"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ListIncidentsSort(str, Enum):
    CREATEDAT = "createdAt"
    ISSEEN = "isSeen"
    MESSAGE = "message"
    OWNER = "owner"
    STATUS = "status"
    TINYID = "tinyId"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BaseIntegrationActionType(str, Enum):
    ACKNOWLEDGE = "acknowledge"
    ADDNOTE = "addNote"
    CLOSE = "close"
    CREATE = "create"
    IGNORE = "ignore"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.integration_condition_operation import IntegrationConditionOperation
from ..types import UNSET, Unset

T = TypeVar("T", bound="IntegrationCondition")


@_attrs_define
class IntegrationCondition:
    """
    Attributes:
        field (Union[Unset, str]):
        not_ (Union[Unset, bool]):
        operation (Union[Unset, IntegrationConditionOperation]):
        expected_value (Union[Unset, str]):
    """

    field: Union[Unset, str] = UNSET
    not_: Union[Unset, bool] = UNSET
    operation: Union[Unset, IntegrationConditionOperation] = UNSET
    expected_value: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field = self.field

        not_ = self.not_

        operation: Union[Unset, str] = UNSET
        if not isinstance(self.operation, Unset):
            operation = self.operation.value

        expected_value = self.expected_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field is not UNSET:
            field_dict["field"] = field
        if not_ is not UNSET:
            field_dict["not"] = not_
        if operation is not UNSET:
            field_dict["operation"] = operation
        if expected_value is not UNSET:
            field_dict["expectedValue"] = expected_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field = d.pop("field", UNSET)

        not_ = d.pop("not", UNSET)

        _operation = d.pop("operation", UNSET)
        operation: Union[Unset, IntegrationConditionOperation]
        if isinstance(_operation, Unset):
            operation = UNSET
        else:
            operation = IntegrationConditionOperation(_operation)

        expected_value = d.pop("expectedValue", UNSET)

        integration_condition = cls(
            field=field,
            not_=not_,
            operation=operation,
            expected_value=expected_value,
        )

        integration_condition.additional_properties = d
        return integration_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.recipient_type import RecipientType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Recipient")


@_attrs_define
class Recipient:
    """
    Attributes:
        type (RecipientType):
        id (Union[Unset, str]):
    """

    type: RecipientType
    id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = RecipientType(d.pop("type"))

        id = d.pop("id", UNSET)

        recipient = cls(
            type=type,
            id=id,
        )

        recipient.additional_properties = d
        return recipient

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.deprecated_alert_policy_type import DeprecatedAlertPolicyType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_ import Filter
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="DeprecatedAlertPolicy")


@_attrs_define
class DeprecatedAlertPolicy:
    """
    Attributes:
        type (DeprecatedAlertPolicyType): Type of the policy
        id (Union[Unset, str]):
        name (Union[Unset, str]): Name of the policy
        policy_description (Union[Unset, str]): Description of the policy
        filter_ (Union[Unset, Filter]): Defines the conditions that will be checked before applying rules and type of
            the operations that will be applied on conditions
        time_restrictions (Union[Unset, TimeRestrictionInterval]):
        enabled (Union[Unset, bool]): Activity status of the alert policy
    """

    type: DeprecatedAlertPolicyType
    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    policy_description: Union[Unset, str] = UNSET
    filter_: Union[Unset, "Filter"] = UNSET
    time_restrictions: Union[Unset, "TimeRestrictionInterval"] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        id = self.id

        name = self.name

        policy_description = self.policy_description

        filter_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filter_, Unset):
            filter_ = self.filter_.to_dict()

        time_restrictions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restrictions, Unset):
            time_restrictions = self.time_restrictions.to_dict()

        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if policy_description is not UNSET:
            field_dict["policyDescription"] = policy_description
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if time_restrictions is not UNSET:
            field_dict["timeRestrictions"] = time_restrictions
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_ import Filter
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        type = DeprecatedAlertPolicyType(d.pop("type"))

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        policy_description = d.pop("policyDescription", UNSET)

        _filter_ = d.pop("filter", UNSET)
        filter_: Union[Unset, Filter]
        if isinstance(_filter_, Unset):
            filter_ = UNSET
        else:
            filter_ = Filter.from_dict(_filter_)

        _time_restrictions = d.pop("timeRestrictions", UNSET)
        time_restrictions: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restrictions, Unset):
            time_restrictions = UNSET
        else:
            time_restrictions = TimeRestrictionInterval.from_dict(_time_restrictions)

        enabled = d.pop("enabled", UNSET)

        deprecated_alert_policy = cls(
            type=type,
            id=id,
            name=name,
            policy_description=policy_description,
            filter_=filter_,
            time_restrictions=time_restrictions,
            enabled=enabled,
        )

        deprecated_alert_policy.additional_properties = d
        return deprecated_alert_policy

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ServiceAudienceTemplateConditionMatchField(str, Enum):
    CITY = "city"
    COUNTRY = "country"
    CUSTOMPROPERTY = "customProperty"
    LINE = "line"
    STATE = "state"
    TAG = "tag"
    ZIPCODE = "zipCode"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BaseWebhookCallbackBaseWebhookCallbackType(str, Enum):
    FLOCK_CALLBACK = "flock-callback"
    KORE_CALLBACK = "kore-callback"
    MOXTRA_CALLBACK = "moxtra-callback"
    RING_CENTRAL_GLIP_CALLBACK = "ring-central-glip-callback"
    STATUSY_CALLBACK = "statusy-callback"
    WEBHOOK_CALLBACK = "webhook-callback"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.service_incident_properties_priority import ServiceIncidentPropertiesPriority
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.service_incident_properties_details import ServiceIncidentPropertiesDetails
    from ..models.service_incident_stakeholder_properties import ServiceIncidentStakeholderProperties


T = TypeVar("T", bound="ServiceIncidentProperties")


@_attrs_define
class ServiceIncidentProperties:
    """
    Attributes:
        message (str): Message of the related incident rule.
        priority (ServiceIncidentPropertiesPriority): Priority level of the alert. Possible values are P1, P2, P3, P4
            and P5
        stakeholder_properties (ServiceIncidentStakeholderProperties):
        tags (Union[Unset, List[str]]): Tags of the alert.
        details (Union[Unset, ServiceIncidentPropertiesDetails]): Map of key-value pairs to use as custom properties of
            the alert.
        description (Union[Unset, str]): Description field of the incident rule.
    """

    message: str
    priority: ServiceIncidentPropertiesPriority
    stakeholder_properties: "ServiceIncidentStakeholderProperties"
    tags: Union[Unset, List[str]] = UNSET
    details: Union[Unset, "ServiceIncidentPropertiesDetails"] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message

        priority = self.priority.value

        stakeholder_properties = self.stakeholder_properties.to_dict()

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
                "priority": priority,
                "stakeholderProperties": stakeholder_properties,
            }
        )
        if tags is not UNSET:
            field_dict["tags"] = tags
        if details is not UNSET:
            field_dict["details"] = details
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.service_incident_properties_details import ServiceIncidentPropertiesDetails
        from ..models.service_incident_stakeholder_properties import ServiceIncidentStakeholderProperties

        d = src_dict.copy()
        message = d.pop("message")

        priority = ServiceIncidentPropertiesPriority(d.pop("priority"))

        stakeholder_properties = ServiceIncidentStakeholderProperties.from_dict(d.pop("stakeholderProperties"))

        tags = cast(List[str], d.pop("tags", UNSET))

        _details = d.pop("details", UNSET)
        details: Union[Unset, ServiceIncidentPropertiesDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = ServiceIncidentPropertiesDetails.from_dict(_details)

        description = d.pop("description", UNSET)

        service_incident_properties = cls(
            message=message,
            priority=priority,
            stakeholder_properties=stakeholder_properties,
            tags=tags,
            details=details,
            description=description,
        )

        service_incident_properties.additional_properties = d
        return service_incident_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.timeline_recipient import TimelineRecipient


T = TypeVar("T", bound="TimelinePeriod")


@_attrs_define
class TimelinePeriod:
    """
    Attributes:
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        type (Union[Unset, str]):
        from_user (Union[Unset, str]): Only used by 'forwarding' period types
        recipient (Union[Unset, TimelineRecipient]):
        flattened_recipients (Union[Unset, List['TimelineRecipient']]): Only used by 'historical' period types
    """

    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    type: Union[Unset, str] = UNSET
    from_user: Union[Unset, str] = UNSET
    recipient: Union[Unset, "TimelineRecipient"] = UNSET
    flattened_recipients: Union[Unset, List["TimelineRecipient"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        type = self.type

        from_user = self.from_user

        recipient: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.recipient, Unset):
            recipient = self.recipient.to_dict()

        flattened_recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.flattened_recipients, Unset):
            flattened_recipients = []
            for flattened_recipients_item_data in self.flattened_recipients:
                flattened_recipients_item = flattened_recipients_item_data.to_dict()
                flattened_recipients.append(flattened_recipients_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if type is not UNSET:
            field_dict["type"] = type
        if from_user is not UNSET:
            field_dict["fromUser"] = from_user
        if recipient is not UNSET:
            field_dict["recipient"] = recipient
        if flattened_recipients is not UNSET:
            field_dict["flattenedRecipients"] = flattened_recipients

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.timeline_recipient import TimelineRecipient

        d = src_dict.copy()
        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        type = d.pop("type", UNSET)

        from_user = d.pop("fromUser", UNSET)

        _recipient = d.pop("recipient", UNSET)
        recipient: Union[Unset, TimelineRecipient]
        if isinstance(_recipient, Unset):
            recipient = UNSET
        else:
            recipient = TimelineRecipient.from_dict(_recipient)

        flattened_recipients = []
        _flattened_recipients = d.pop("flattenedRecipients", UNSET)
        for flattened_recipients_item_data in _flattened_recipients or []:
            flattened_recipients_item = TimelineRecipient.from_dict(flattened_recipients_item_data)

            flattened_recipients.append(flattened_recipients_item)

        timeline_period = cls(
            start_date=start_date,
            end_date=end_date,
            type=type,
            from_user=from_user,
            recipient=recipient,
            flattened_recipients=flattened_recipients,
        )

        timeline_period.additional_properties = d
        return timeline_period

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ChangeTeamRoutingRuleOrderPayload")


@_attrs_define
class ChangeTeamRoutingRuleOrderPayload:
    """
    Attributes:
        order (Union[Unset, int]): Order of team routing rule within the rules. order value is actually the index of the
            team routing rule.
    """

    order: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        order = self.order

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        order = d.pop("order", UNSET)

        change_team_routing_rule_order_payload = cls(
            order=order,
        )

        change_team_routing_rule_order_payload.additional_properties = d
        return change_team_routing_rule_order_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.team_member import TeamMember


T = TypeVar("T", bound="CreateTeamPayload")


@_attrs_define
class CreateTeamPayload:
    """
    Attributes:
        name (str): Name of the team
        description (Union[Unset, str]): The description of team
        members (Union[Unset, List['TeamMember']]): The users which will be added to team, and optionally their roles.
    """

    name: str
    description: Union[Unset, str] = UNSET
    members: Union[Unset, List["TeamMember"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        members: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.members, Unset):
            members = []
            for members_item_data in self.members:
                members_item = members_item_data.to_dict()
                members.append(members_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if members is not UNSET:
            field_dict["members"] = members

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.team_member import TeamMember

        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description", UNSET)

        members = []
        _members = d.pop("members", UNSET)
        for members_item_data in _members or []:
            members_item = TeamMember.from_dict(members_item_data)

            members.append(members_item)

        create_team_payload = cls(
            name=name,
            description=description,
            members=members,
        )

        create_team_payload.additional_properties = d
        return create_team_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DeprecatedAlertPolicyType(str, Enum):
    AUTO_CLOSE = "auto-close"
    AUTO_RESTART_NOTIFICATIONS = "auto-restart-notifications"
    MODIFY = "modify"
    NOTIFICATION_DEDUPLICATION = "notification-deduplication"
    NOTIFICATION_DELAY = "notification-delay"
    NOTIFICATION_RENOTIFY = "notification-renotify"
    NOTIFICATION_SUPPRESS = "notification-suppress"

    def __str__(self) -> str:
        return str(self.value)



import json
import os
import pathlib
import shutil
from collections import OrderedDict

CONFIG_DIR = pathlib.Path.home() / ".config" / "tygenie"
CONFIG_FILE = "tygenie.json"
CONFIG_PATH = CONFIG_DIR / CONFIG_FILE
SAMPLE_CONFIG = f"{os.path.dirname(__file__)}/assets/tygenie.json"


class Config:
    def __init__(self, config_path: pathlib.Path | str = CONFIG_PATH):
        self.config_dir: pathlib.Path = CONFIG_DIR
        self.config_file: str = CONFIG_FILE
        self._config_path: pathlib.Path = pathlib.Path(config_path)
        self.config: dict = {}
        self.tygenie: dict = {}
        self.opsgenie: dict = {}
        self.sample_copied: bool = False

        super().__init__()
        self.__init__config()
        self.auto_update()

    @property
    def config_path(self):
        return self._config_path

    @config_path.setter
    def config_path(self, value: pathlib.Path | str = CONFIG_PATH):
        if isinstance(value, str):
            value = pathlib.Path(value)

        self._config_path = value

    def __init__config(self):
        os.makedirs(self.config_dir, exist_ok=True)
        if not os.path.exists(self.config_path):
            shutil.copyfile(SAMPLE_CONFIG, self.config_path)
            self.sample_copied = True

    def _load_config(self):
        self.tygenie = self.config.get("tygenie", {})
        self.opsgenie = self.config.get("opsgenie", {})

    def load(self):
        with open(self.config_path, "r") as conf:
            self.config = json.load(conf, object_pairs_hook=OrderedDict)

        self._load_config()

    def reload(self):
        self.load()

    def save(self, config):
        with open(self.config_path, "w") as conf:
            conf.write(json.dumps(config, indent=2))

        self.config = config
        self._load_config()

    def auto_update(self):
        """This method automatically update deprecated parameters"""

        self.load()

        # Update desktop notification parmeters, was "notify" before "desktop_notification"
        notify = self.config.get("tygenie", {}).get("notify", None)
        dn = self.config.get("tygenie", {}).get("desktop_notification", {})
        dn_urgency = self.config.get("tygenie", {}).get(
            "desktop_notification_urgency", "Critical"
        )
        # In case we have notify but desktop_notification parameter we force dn to
        # be notify value to pass in the next if
        if notify is not None and dn == {}:
            dn = notify

        if type(dn) is bool:  # either we have notify or the depreccated bool value
            new_dn = {
                "enable": dn,
                "urgency": dn_urgency,
                "when_on_call_only": False,
            }
            self.config.get("tygenie", {})["desktop_notification"] = new_dn
            for key in ["notify", "desktop_notification_urgency"]:
                self.config.get("tygenie", {}).pop(key, None)

        # Update deprecated plugins parameters
        plugins = self.config.get("tygenie", {})["plugins"]
        if (
            "alerts_list_formatter" not in plugins
            and "alert_description_formatter" not in plugins
        ):

            new_plugins_config = {
                "alerts_list_formatter": plugins.get("alert_formatter", None),
                "alert_description_formatter": plugins.get("content_transformer", None),
            }
            self.config.get("tygenie", {})["plugins"] = new_plugins_config

        # Add theme parameter config
        self.config.get("tygenie", {})["theme"] = self.config.get("tygenie", {}).get(
            "theme", "flexoki"
        )

        # Add palette keybinding
        self.config.get("tygenie", {}).get("keybindings", {})["palette"] = (
            self.config.get("tygenie", {})
            .get("keybindings", {})
            .get("palette", "ctrl+p")
        )

        # Add displayed_field as a key/value structure
        self.config.get("tygenie", {})["displayed_fields"] = self.config.get(
            "tygenie", {}
        ).get("displayed_fields", {})

        self.save(self.config)


ty_config = Config()
ty_config.load()


def set_config_file(config_file):
    global ty_config
    ty_config = Config(config_path=config_file)
    ty_config.load()



from enum import Enum


class ActionMappingMappedAction(str, Enum):
    ACKNOWLEDGE = "acknowledge"
    ACKNOWLEDGESENSOR = "acknowledgeSensor"
    ACKNOWLEDGE_ALERT = "acknowledge-alert"
    ACKNOWLEDGE_NOTIFICATION = "acknowledge-notification"
    ACK_ALERT = "ack-alert"
    ADD_COMMENT_TO_ISSUE = "add-comment-to-issue"
    ADD_DISCUSSION_NOTE_TO_TICKET = "add-discussion-note-to-ticket"
    ADD_INTERNAL_NOTE = "add-internal-note"
    ADD_INTERNAL_NOTE_TO_TICKET = "add-internal-note-to-ticket"
    ADD_JOURNAL = "add-journal"
    ADD_NOTE = "add-note"
    ADD_NOTE_TO_INCIDENT = "add-note-to-incident"
    ADD_PRIVATE_NOTE = "add-private-note"
    ADD_PUBLIC_NOTE = "add-public-note"
    ADD_RESOLUTION_NOTE_TO_TICKET = "add-resolution-note-to-ticket"
    ADD_TAG = "add-tag"
    ADD_WORK_INFO = "add-work-info"
    ASSIGN_TO_GROUP = "assign-to-group"
    ASSIGN_TO_OWNER = "assign-to-owner"
    ASSIGN_TO_USER = "assign-to-user"
    CLOSE = "close"
    CLOSE_ALERT = "close-alert"
    CLOSE_INCIDENT = "close-incident"
    CLOSE_ISSUE = "close-issue"
    CLOSE_SERVICE_TICKET = "close-service-ticket"
    CONFIRM_INCIDENT = "confirm-incident"
    CREATE = "create"
    CREATECOMMENT = "createComment"
    CREATEEVENT = "createEvent"
    CREATETAG = "createTag"
    CREATE_ALERT = "create-alert"
    CREATE_CASE = "create-case"
    CREATE_CONVERSATION = "create-conversation"
    CREATE_INCIDENT = "create-incident"
    CREATE_INCIDENT_AND_UPDATE_COMPONENT_STATUSES = "create-incident-and-update-component-statuses"
    CREATE_ISSUE = "create-issue"
    CREATE_PROBLEM = "create-problem"
    CREATE_SERVICE_TICKET = "create-service-ticket"
    CREATE_TICKET = "create-ticket"
    CUSTOM_STATUS = "custom-status"
    IN_PROGRESS_INCIDENT = "in-progress-incident"
    IN_PROGRESS_ISSUE = "in-progress-issue"
    IN_PROGRESS_TICKET = "in-progress-ticket"
    ISSUE_DONE = "issue-done"
    POSTTOURL = "postToUrl"
    POST_ACTION_TO_OEC = "post-action-to-oec"
    PUBLIC_REPLY = "public-reply"
    PUBLISH = "publish"
    RESOLVE = "resolve"
    RESOLVE_INCIDENT = "resolve-incident"
    RESOLVE_INCIDENT_AND_UPDATE_COMPONENT_STATUSES_BACK_TO_OPERATIONAL = (
        "resolve-incident-and-update-component-statuses-back-to-operational"
    )
    RESOLVE_ISSUE = "resolve-issue"
    RESOLVE_TICKET = "resolve-ticket"
    SENDDATA = "sendData"
    SEND_TO_CHANNEL = "send-to-channel"
    SEND_TO_CUSTOM_ENDPOINT = "send-to-custom-endpoint"
    SETSTATUSTOCLOSED = "setStatusToClosed"
    SET_CASE_STATUS_TO_ACTIVE = "set-case-status-to-active"
    SET_CASE_STATUS_TO_CLOSE = "set-case-status-to-close"
    SET_CASE_STATUS_TO_CLOSE_AS_DUPLICATE = "set-case-status-to-close-as-duplicate"
    SET_CASE_STATUS_TO_WORKING = "set-case-status-to-working"
    SET_STATUS_TO_CLOSED = "set-status-to-closed"
    SET_STATUS_TO_COMPLETED = "set-status-to-completed"
    SET_STATUS_TO_OPEN = "set-status-to-open"
    SET_STATUS_TO_PENDING = "set-status-to-pending"
    SET_STATUS_TO_RESOLVED = "set-status-to-resolved"
    SET_STATUS_TO_SOLVED = "set-status-to-solved"
    UNACKNOWLEDGE = "unacknowledge"
    UNACKNOWLEDGE_ALERT = "unacknowledge-alert"
    UNMUTE_ALERT = "unmute-alert"
    UPDATESEVERITY = "updateSeverity"
    UPDATESTATUSTOCLOSED = "updateStatusToClosed"
    UPDATESTATUSTOINPROGRESS = "updateStatusToInProgress"
    UPDATE_COMPONENT_STATUSES = "update-component-statuses"
    UPDATE_DESCRIPTION = "update-description"
    UPDATE_PRIORITY = "update-priority"

    def __str__(self) -> str:
        return str(self.value)



from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Grid
from textual.screen import ModalScreen
from textual.widgets import Button, Footer, Label, TextArea


class AddNoteScreen(ModalScreen):
    """Screen with a textbox to add a note to an alert"""

    BINDINGS = [
        Binding(
            key="escape",
            action="close_modal()",
            description="Close",
            key_display="<esc>",
        ),
    ]

    def compose(self) -> ComposeResult:
        yield Label("[b]Add a note to alert[/b]")
        message = ""
        textarea = TextArea(message, language=None, id="add_note_textarea")
        textarea.language = "markdown"
        textarea.show_line_numbers = True
        yield Grid(
            textarea,
            Button("Cancel", variant="error", id="cancel"),
            Button("Add", variant="success", id="add"),
            id="add_note_grid",
        )
        yield Footer()

    def action_close_modal(self, note: None | str = None):
        textarea = self.query_one("#add_note_textarea", TextArea)
        textarea.focus(True)
        textarea.clear()
        self.dismiss(result=note)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        note = ""
        if event.button.id == "add":
            textarea = self.query_one("#add_note_textarea", TextArea)
            note = textarea.text

        self.action_close_modal(note=note)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.policy_type import PolicyType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.auto_close_action import AutoCloseAction
    from ..models.auto_restart_action import AutoRestartAction
    from ..models.deduplication_action import DeduplicationAction
    from ..models.delay_action import DelayAction
    from ..models.filter_ import Filter
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="NotificationPolicy")


@_attrs_define
class NotificationPolicy:
    """
    Attributes:
        type (PolicyType): Type of the policy
        id (Union[Unset, str]):
        name (Union[Unset, str]): Name of the policy
        policy_description (Union[Unset, str]): Description of the policy
        team_id (Union[Unset, str]): TeamId of the policy
        filter_ (Union[Unset, Filter]): Defines the conditions that will be checked before applying rules and type of
            the operations that will be applied on conditions
        time_restrictions (Union[Unset, TimeRestrictionInterval]):
        enabled (Union[Unset, bool]): Activity status of the alert policy
        auto_restart_action (Union[Unset, AutoRestartAction]):
        auto_close_action (Union[Unset, AutoCloseAction]):
        deduplication_action (Union[Unset, DeduplicationAction]):
        delay_action (Union[Unset, DelayAction]):
        suppress (Union[Unset, bool]): If set to true, notifications related to the matching alert will be suppressed.
            Default value is false.
    """

    type: PolicyType
    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    policy_description: Union[Unset, str] = UNSET
    team_id: Union[Unset, str] = UNSET
    filter_: Union[Unset, "Filter"] = UNSET
    time_restrictions: Union[Unset, "TimeRestrictionInterval"] = UNSET
    enabled: Union[Unset, bool] = UNSET
    auto_restart_action: Union[Unset, "AutoRestartAction"] = UNSET
    auto_close_action: Union[Unset, "AutoCloseAction"] = UNSET
    deduplication_action: Union[Unset, "DeduplicationAction"] = UNSET
    delay_action: Union[Unset, "DelayAction"] = UNSET
    suppress: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        id = self.id

        name = self.name

        policy_description = self.policy_description

        team_id = self.team_id

        filter_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filter_, Unset):
            filter_ = self.filter_.to_dict()

        time_restrictions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restrictions, Unset):
            time_restrictions = self.time_restrictions.to_dict()

        enabled = self.enabled

        auto_restart_action: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_restart_action, Unset):
            auto_restart_action = self.auto_restart_action.to_dict()

        auto_close_action: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_close_action, Unset):
            auto_close_action = self.auto_close_action.to_dict()

        deduplication_action: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.deduplication_action, Unset):
            deduplication_action = self.deduplication_action.to_dict()

        delay_action: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.delay_action, Unset):
            delay_action = self.delay_action.to_dict()

        suppress = self.suppress

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if policy_description is not UNSET:
            field_dict["policyDescription"] = policy_description
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if time_restrictions is not UNSET:
            field_dict["timeRestrictions"] = time_restrictions
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if auto_restart_action is not UNSET:
            field_dict["autoRestartAction"] = auto_restart_action
        if auto_close_action is not UNSET:
            field_dict["autoCloseAction"] = auto_close_action
        if deduplication_action is not UNSET:
            field_dict["deduplicationAction"] = deduplication_action
        if delay_action is not UNSET:
            field_dict["delayAction"] = delay_action
        if suppress is not UNSET:
            field_dict["suppress"] = suppress

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.auto_close_action import AutoCloseAction
        from ..models.auto_restart_action import AutoRestartAction
        from ..models.deduplication_action import DeduplicationAction
        from ..models.delay_action import DelayAction
        from ..models.filter_ import Filter
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        type = PolicyType(d.pop("type"))

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        policy_description = d.pop("policyDescription", UNSET)

        team_id = d.pop("teamId", UNSET)

        _filter_ = d.pop("filter", UNSET)
        filter_: Union[Unset, Filter]
        if isinstance(_filter_, Unset):
            filter_ = UNSET
        else:
            filter_ = Filter.from_dict(_filter_)

        _time_restrictions = d.pop("timeRestrictions", UNSET)
        time_restrictions: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restrictions, Unset):
            time_restrictions = UNSET
        else:
            time_restrictions = TimeRestrictionInterval.from_dict(_time_restrictions)

        enabled = d.pop("enabled", UNSET)

        _auto_restart_action = d.pop("autoRestartAction", UNSET)
        auto_restart_action: Union[Unset, AutoRestartAction]
        if isinstance(_auto_restart_action, Unset):
            auto_restart_action = UNSET
        else:
            auto_restart_action = AutoRestartAction.from_dict(_auto_restart_action)

        _auto_close_action = d.pop("autoCloseAction", UNSET)
        auto_close_action: Union[Unset, AutoCloseAction]
        if isinstance(_auto_close_action, Unset):
            auto_close_action = UNSET
        else:
            auto_close_action = AutoCloseAction.from_dict(_auto_close_action)

        _deduplication_action = d.pop("deduplicationAction", UNSET)
        deduplication_action: Union[Unset, DeduplicationAction]
        if isinstance(_deduplication_action, Unset):
            deduplication_action = UNSET
        else:
            deduplication_action = DeduplicationAction.from_dict(_deduplication_action)

        _delay_action = d.pop("delayAction", UNSET)
        delay_action: Union[Unset, DelayAction]
        if isinstance(_delay_action, Unset):
            delay_action = UNSET
        else:
            delay_action = DelayAction.from_dict(_delay_action)

        suppress = d.pop("suppress", UNSET)

        notification_policy = cls(
            type=type,
            id=id,
            name=name,
            policy_description=policy_description,
            team_id=team_id,
            filter_=filter_,
            time_restrictions=time_restrictions,
            enabled=enabled,
            auto_restart_action=auto_restart_action,
            auto_close_action=auto_close_action,
            deduplication_action=deduplication_action,
            delay_action=delay_action,
            suppress=suppress,
        )

        notification_policy.additional_properties = d
        return notification_policy

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Participant")


@_attrs_define
class Participant:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        type (Union[Unset, str]):
        on_call_participants (Union[Unset, List['Participant']]):
        forwarded_from (Union[Unset, Participant]):
        escalation_time (Union[Unset, int]): Only used by 'escalation' participants
        notify_type (Union[Unset, str]): Only used by 'escalation' participants
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    on_call_participants: Union[Unset, List["Participant"]] = UNSET
    forwarded_from: Union[Unset, "Participant"] = UNSET
    escalation_time: Union[Unset, int] = UNSET
    notify_type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        type = self.type

        on_call_participants: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.on_call_participants, Unset):
            on_call_participants = []
            for on_call_participants_item_data in self.on_call_participants:
                on_call_participants_item = on_call_participants_item_data.to_dict()
                on_call_participants.append(on_call_participants_item)

        forwarded_from: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.forwarded_from, Unset):
            forwarded_from = self.forwarded_from.to_dict()

        escalation_time = self.escalation_time

        notify_type = self.notify_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type
        if on_call_participants is not UNSET:
            field_dict["onCallParticipants"] = on_call_participants
        if forwarded_from is not UNSET:
            field_dict["forwardedFrom"] = forwarded_from
        if escalation_time is not UNSET:
            field_dict["escalationTime"] = escalation_time
        if notify_type is not UNSET:
            field_dict["notifyType"] = notify_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        type = d.pop("type", UNSET)

        on_call_participants = []
        _on_call_participants = d.pop("onCallParticipants", UNSET)
        for on_call_participants_item_data in _on_call_participants or []:
            on_call_participants_item = Participant.from_dict(on_call_participants_item_data)

            on_call_participants.append(on_call_participants_item)

        _forwarded_from = d.pop("forwardedFrom", UNSET)
        forwarded_from: Union[Unset, Participant]
        if isinstance(_forwarded_from, Unset):
            forwarded_from = UNSET
        else:
            forwarded_from = Participant.from_dict(_forwarded_from)

        escalation_time = d.pop("escalationTime", UNSET)

        notify_type = d.pop("notifyType", UNSET)

        participant = cls(
            id=id,
            name=name,
            type=type,
            on_call_participants=on_call_participants,
            forwarded_from=forwarded_from,
            escalation_time=escalation_time,
            notify_type=notify_type,
        )

        participant.additional_properties = d
        return participant

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.update_heartbeat_payload_interval_unit import UpdateHeartbeatPayloadIntervalUnit
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateHeartbeatPayload")


@_attrs_define
class UpdateHeartbeatPayload:
    """
    Attributes:
        description (Union[Unset, str]): An optional description of the heartbeat
        interval (Union[Unset, int]): Specifies how often a heartbeat message should be expected
        interval_unit (Union[Unset, UpdateHeartbeatPayloadIntervalUnit]): Interval specified as 'minutes', 'hours' or
            'days'
        enabled (Union[Unset, bool]): Enable/disable heartbeat monitoring
    """

    description: Union[Unset, str] = UNSET
    interval: Union[Unset, int] = UNSET
    interval_unit: Union[Unset, UpdateHeartbeatPayloadIntervalUnit] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        description = self.description

        interval = self.interval

        interval_unit: Union[Unset, str] = UNSET
        if not isinstance(self.interval_unit, Unset):
            interval_unit = self.interval_unit.value

        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description
        if interval is not UNSET:
            field_dict["interval"] = interval
        if interval_unit is not UNSET:
            field_dict["intervalUnit"] = interval_unit
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        description = d.pop("description", UNSET)

        interval = d.pop("interval", UNSET)

        _interval_unit = d.pop("intervalUnit", UNSET)
        interval_unit: Union[Unset, UpdateHeartbeatPayloadIntervalUnit]
        if isinstance(_interval_unit, Unset):
            interval_unit = UNSET
        else:
            interval_unit = UpdateHeartbeatPayloadIntervalUnit(_interval_unit)

        enabled = d.pop("enabled", UNSET)

        update_heartbeat_payload = cls(
            description=description,
            interval=interval,
            interval_unit=interval_unit,
            enabled=enabled,
        )

        update_heartbeat_payload.additional_properties = d
        return update_heartbeat_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.service_audience_template_responder import ServiceAudienceTemplateResponder
    from ..models.service_audience_template_stakeholder import ServiceAudienceTemplateStakeholder


T = TypeVar("T", bound="ServiceAudienceTemplate")


@_attrs_define
class ServiceAudienceTemplate:
    """
    Attributes:
        responder (Union[Unset, ServiceAudienceTemplateResponder]):
        stakeholder (Union[Unset, ServiceAudienceTemplateStakeholder]):
    """

    responder: Union[Unset, "ServiceAudienceTemplateResponder"] = UNSET
    stakeholder: Union[Unset, "ServiceAudienceTemplateStakeholder"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        responder: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.responder, Unset):
            responder = self.responder.to_dict()

        stakeholder: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stakeholder, Unset):
            stakeholder = self.stakeholder.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if responder is not UNSET:
            field_dict["responder"] = responder
        if stakeholder is not UNSET:
            field_dict["stakeholder"] = stakeholder

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.service_audience_template_responder import ServiceAudienceTemplateResponder
        from ..models.service_audience_template_stakeholder import ServiceAudienceTemplateStakeholder

        d = src_dict.copy()
        _responder = d.pop("responder", UNSET)
        responder: Union[Unset, ServiceAudienceTemplateResponder]
        if isinstance(_responder, Unset):
            responder = UNSET
        else:
            responder = ServiceAudienceTemplateResponder.from_dict(_responder)

        _stakeholder = d.pop("stakeholder", UNSET)
        stakeholder: Union[Unset, ServiceAudienceTemplateStakeholder]
        if isinstance(_stakeholder, Unset):
            stakeholder = UNSET
        else:
            stakeholder = ServiceAudienceTemplateStakeholder.from_dict(_stakeholder)

        service_audience_template = cls(
            responder=responder,
            stakeholder=stakeholder,
        )

        service_audience_template.additional_properties = d
        return service_audience_template

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.list_incidents_order import ListIncidentsOrder
from ...models.list_incidents_response import ListIncidentsResponse
from ...models.list_incidents_sort import ListIncidentsSort
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    query: str,
    offset: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListIncidentsSort] = ListIncidentsSort.CREATEDAT,
    order: Union[Unset, ListIncidentsOrder] = ListIncidentsOrder.DESC,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["query"] = query

    params["offset"] = offset

    params["limit"] = limit

    json_sort: Union[Unset, str] = UNSET
    if not isinstance(sort, Unset):
        json_sort = sort.value

    params["sort"] = json_sort

    json_order: Union[Unset, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value

    params["order"] = json_order

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/incidents/",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, ListIncidentsResponse]]:
    if response.status_code == 200:
        response_200 = ListIncidentsResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 402:
        response_402 = cast(Any, None)
        return response_402
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 422:
        response_422 = cast(Any, None)
        return response_422
    if response.status_code == 429:
        response_429 = cast(Any, None)
        return response_429
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, ListIncidentsResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    query: str,
    offset: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListIncidentsSort] = ListIncidentsSort.CREATEDAT,
    order: Union[Unset, ListIncidentsOrder] = ListIncidentsOrder.DESC,
) -> Response[Union[Any, ListIncidentsResponse]]:
    """List incidents

     Return list of incidents

    Args:
        query (str):
        offset (Union[Unset, int]):
        limit (Union[Unset, int]):
        sort (Union[Unset, ListIncidentsSort]):  Default: ListIncidentsSort.CREATEDAT.
        order (Union[Unset, ListIncidentsOrder]):  Default: ListIncidentsOrder.DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListIncidentsResponse]]
    """

    kwargs = _get_kwargs(
        query=query,
        offset=offset,
        limit=limit,
        sort=sort,
        order=order,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    query: str,
    offset: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListIncidentsSort] = ListIncidentsSort.CREATEDAT,
    order: Union[Unset, ListIncidentsOrder] = ListIncidentsOrder.DESC,
) -> Optional[Union[Any, ListIncidentsResponse]]:
    """List incidents

     Return list of incidents

    Args:
        query (str):
        offset (Union[Unset, int]):
        limit (Union[Unset, int]):
        sort (Union[Unset, ListIncidentsSort]):  Default: ListIncidentsSort.CREATEDAT.
        order (Union[Unset, ListIncidentsOrder]):  Default: ListIncidentsOrder.DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListIncidentsResponse]
    """

    return sync_detailed(
        client=client,
        query=query,
        offset=offset,
        limit=limit,
        sort=sort,
        order=order,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    query: str,
    offset: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListIncidentsSort] = ListIncidentsSort.CREATEDAT,
    order: Union[Unset, ListIncidentsOrder] = ListIncidentsOrder.DESC,
) -> Response[Union[Any, ListIncidentsResponse]]:
    """List incidents

     Return list of incidents

    Args:
        query (str):
        offset (Union[Unset, int]):
        limit (Union[Unset, int]):
        sort (Union[Unset, ListIncidentsSort]):  Default: ListIncidentsSort.CREATEDAT.
        order (Union[Unset, ListIncidentsOrder]):  Default: ListIncidentsOrder.DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ListIncidentsResponse]]
    """

    kwargs = _get_kwargs(
        query=query,
        offset=offset,
        limit=limit,
        sort=sort,
        order=order,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    query: str,
    offset: Union[Unset, int] = UNSET,
    limit: Union[Unset, int] = UNSET,
    sort: Union[Unset, ListIncidentsSort] = ListIncidentsSort.CREATEDAT,
    order: Union[Unset, ListIncidentsOrder] = ListIncidentsOrder.DESC,
) -> Optional[Union[Any, ListIncidentsResponse]]:
    """List incidents

     Return list of incidents

    Args:
        query (str):
        offset (Union[Unset, int]):
        limit (Union[Unset, int]):
        sort (Union[Unset, ListIncidentsSort]):  Default: ListIncidentsSort.CREATEDAT.
        order (Union[Unset, ListIncidentsOrder]):  Default: ListIncidentsOrder.DESC.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ListIncidentsResponse]
    """

    return (
        await asyncio_detailed(
            client=client,
            query=query,
            offset=offset,
            limit=limit,
            sort=sort,
            order=order,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.maintenance_rule_state import MaintenanceRuleState
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.maintenance_entity import MaintenanceEntity


T = TypeVar("T", bound="MaintenanceRule")


@_attrs_define
class MaintenanceRule:
    """
    Attributes:
        entity (MaintenanceEntity):
        state (Union[Unset, MaintenanceRuleState]): Defines the state of the rule
    """

    entity: "MaintenanceEntity"
    state: Union[Unset, MaintenanceRuleState] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        entity = self.entity.to_dict()

        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "entity": entity,
            }
        )
        if state is not UNSET:
            field_dict["state"] = state

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.maintenance_entity import MaintenanceEntity

        d = src_dict.copy()
        entity = MaintenanceEntity.from_dict(d.pop("entity"))

        _state = d.pop("state", UNSET)
        state: Union[Unset, MaintenanceRuleState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = MaintenanceRuleState(_state)

        maintenance_rule = cls(
            entity=entity,
            state=state,
        )

        maintenance_rule.additional_properties = d
        return maintenance_rule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ActionMappingAction(str, Enum):
    ACKNOWLEDGE = "acknowledge"
    ADD_NOTE = "add-note"
    ADD_RESPONDER = "add-responder"
    ADD_TAGS = "add-tags"
    ASSIGN_OWNERSHIP = "assign-ownership"
    CLOSE = "close"
    CREATE = "create"
    CUSTOM_ACTION = "custom-action"
    DELETE = "delete"
    ESCALATE = "escalate"
    ESCALATE_TO_NEXT = "escalate-to-next"
    REMOVE_TAGS = "remove-tags"
    SNOOZE = "snooze"
    TAKE_OWNERSHIP = "take-ownership"
    UNACKNOWLEDGE = "unacknowledge"
    UPDATE_DESCRIPTION = "update-description"
    UPDATE_MESSAGE = "update-message"
    UPDATE_PRIORITY = "update-priority"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CreateContactPayloadMethod(str, Enum):
    EMAIL = "email"
    MOBILE = "mobile"
    SMS = "sms"
    VOICE = "voice"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_meta import UserMeta


T = TypeVar("T", bound="TeamMember")


@_attrs_define
class TeamMember:
    """
    Attributes:
        user (Union[Unset, UserMeta]):
        role (Union[Unset, str]): Member role of the user, consisting 'user', 'admin' or a custom team role. Default
            value is 'user' Default: 'user'.
    """

    user: Union[Unset, "UserMeta"] = UNSET
    role: Union[Unset, str] = "user"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        role = self.role

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if role is not UNSET:
            field_dict["role"] = role

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_meta import UserMeta

        d = src_dict.copy()
        _user = d.pop("user", UNSET)
        user: Union[Unset, UserMeta]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = UserMeta.from_dict(_user)

        role = d.pop("role", UNSET)

        team_member = cls(
            user=user,
            role=role,
        )

        team_member.additional_properties = d
        return team_member

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class IntegrationType(str, Enum):
    AIRBRAKE = "Airbrake"
    ALERTLOGIC = "AlertLogic"
    ALERTSITE = "AlertSite"
    AMAZONCLOUDTRAIL = "AmazonCloudTrail"
    AMAZONEC2AUTOSCALING = "AmazonEc2AutoScaling"
    AMAZONRDS = "AmazonRds"
    AMAZONROUTE53HEALTHCHECK = "AmazonRoute53HealthCheck"
    AMAZONSECURITYHUB = "AmazonSecurityHub"
    AMAZONSES = "AmazonSes"
    AMAZONSNS = "AmazonSns"
    AMAZONSNSOUTGOING = "AmazonSnsOutgoing"
    API = "API"
    APICA = "Apica"
    APIMETRICS = "Apimetrics"
    APPDYNAMICS = "AppDynamics"
    APPOPTICS = "AppOptics"
    APPSIGNAL = "AppSignal"
    APPSIGNALV2 = "AppSignalV2"
    ATATUS = "Atatus"
    ATLASSIANBAMBOOEMAIL = "AtlassianBambooEmail"
    AUTOTASKAEMEMAIL = "AutotaskAEMEmail"
    AUTOTASKEMAIL = "AutoTaskEmail"
    AZURE = "Azure"
    AZUREAUTOSCALE = "AzureAutoScale"
    AZUREOMS = "AzureOMS"
    AZURERESOURCEHEALTH = "AzureResourceHealth"
    AZURESERVICEHEALTH = "AzureServiceHealth"
    BIGPANDA = "BigPanda"
    BITBUCKET = "Bitbucket"
    BLUEMATADOR = "BlueMatador"
    BMCFOOTPRINTSV11 = "BMCFootPrintsV11"
    BMCFOOTPRINTSV12 = "BMCFootPrintsV12"
    BMCREMEDY = "BMCRemedy"
    BMCREMEDYFORCE = "BMCRemedyForce"
    BMCREMEDYONDEMAND = "BMCRemedyOnDemand"
    BOUNDARY = "Boundary"
    CAMPFIRE = "Campfire"
    CATCHPOINT = "Catchpoint"
    CHECKMK = "CheckMK"
    CHERWELL = "Cherwell"
    CIRCLECI = "CircleCi"
    CIRCONUS = "Circonus"
    CLOUDMONIX = "CloudMonix"
    CLOUDSPLOIT = "CloudSploit"
    CLOUDWATCH = "CloudWatch"
    CLOUDWATCHEVENTS = "CloudWatchEvents"
    CODESHIP = "Codeship"
    COMPOSE = "Compose"
    CONNECTWISE = "ConnectWise"
    CONNECTWISEAUTOMATE = "ConnectWiseAutomate"
    CONNECTWISEMANAGE = "ConnectWiseManage"
    CONNECTWISEMANAGEV2 = "ConnectWiseManageV2"
    CONSUL = "Consul"
    COPPEREGG = "CopperEgg"
    CRASHLYTICS = "Crashlytics"
    DATADOG = "Datadog"
    DATALOOPIO = "DataloopIO"
    DESK = "Desk"
    DETECTIFY = "Detectify"
    DNSCHECK = "DNSCheck"
    DRIPSTAT = "DripStat"
    DYNATRACEAPPMON = "DynatraceAppMon"
    DYNATRACEV2 = "DynatraceV2"
    EMAIL = "Email"
    ERRORCEPTION = "Errorception"
    ESWATCHER = "ESWatcher"
    EVIDENTIO = "EvidentIO"
    FLOCK = "Flock"
    FLOWDOCK = "Flowdock"
    FLOWDOCKV2 = "FlowdockV2"
    FRESHDESK = "Freshdesk"
    FRESHSERVICE = "Freshservice"
    GHOSTINSPECTOR = "GhostInspector"
    GITHUB = "GitHub"
    GITLAB = "GitLab"
    GOOGLESTACKDRIVER = "GoogleStackdriver"
    GRAFANA = "Grafana"
    GRAFANAV2 = "GrafanaV2"
    GRAYLOG = "Graylog"
    HEARTBEAT = "Heartbeat"
    HIPCHAT = "HipChat"
    HIPCHATADDON = "HipChatAddOn"
    HIPCHATV2 = "HipChatV2"
    HONEYBADGER = "Honeybadger"
    HOSTEDGRAPHITE = "HostedGraphite"
    HPSERVICEMANAGER = "HPServiceManager"
    HUMIO = "Humio"
    ICINGA = "Icinga"
    ICINGA2 = "Icinga2"
    INCOMINGCALL = "IncomingCall"
    INSTANA = "Instana"
    JENKINS = "Jenkins"
    JIRA = "Jira"
    JIRASERVICEDESK = "JiraServiceDesk"
    KAPACITOR = "Kapacitor"
    KAYAKO = "Kayako"
    KORE = "Kore"
    LABTECHEMAIL = "LabTechEmail"
    LIBRATO = "Librato"
    LIBRENMS = "LibreNMS"
    LIGHTSTEP = "Lightstep"
    LOGENTRIES = "Logentries"
    LOGGLY = "Loggly"
    LOGICMONITOR = "LogicMonitor"
    LOGSTASH = "Logstash"
    LOGZIO = "LogzIO"
    LOOKER = "Looker"
    LOOM = "Loom"
    MAGENTRIX = "Magentrix"
    MANAGEENGINE = "ManageEngine"
    MARID = "Marid"
    MATTERMOST = "Mattermost"
    MONGODBCLOUD = "MongoDBCloud"
    MONITIS = "Monitis"
    MONITISEMAIL = "MonitisEmail"
    MOXTRA = "Moxtra"
    MSTEAMS = "MSTeams"
    MSTEAMSV2 = "MSTeamsV2"
    NAGIOS = "Nagios"
    NAGIOSV2 = "NagiosV2"
    NAGIOSXI = "NagiosXI"
    NAGIOSXIV2 = "NagiosXIV2"
    NETUITIVE = "Netuitive"
    NEUSTAREMAIL = "NeustarEmail"
    NEWRELIC = "NewRelic"
    NEWRELICV2 = "NewRelicV2"
    NODEPING = "NodePing"
    OBSERVIUM = "Observium"
    OBSERVIUMV2 = "ObserviumV2"
    OEC = "OEC"
    OEM = "OEM"
    OEMEMAIL = "OEMEmail"
    OP5 = "OP5"
    OPSDASH = "OpsDash"
    OPSGENIE = "OpsGenie"
    OPSVIEW = "Opsview"
    PAGERDUTYCOMPATIBILITY = "PagerDutyCompatibility"
    PANOPTA = "Panopta"
    PAPERTRAIL = "Papertrail"
    PINGDOM = "Pingdom"
    PINGDOMV2 = "PingdomV2"
    PINGDOMWEBHOOK = "PingdomWebhook"
    PINGOMETER = "Pingometer"
    PLANIO = "Planio"
    PROMETHEUS = "Prometheus"
    PRTG = "Prtg"
    RACKSPACE = "Rackspace"
    RAYGUN = "Raygun"
    REDGATESQLMONITOREMAIL = "RedGateSqlMonitorEmail"
    RIEMANN = "Riemann"
    RIGOR = "Rigor"
    RINGCENTRALEMAIL = "RingCentralEmail"
    RINGCENTRALGLIP = "RingCentralGlip"
    ROLLBAR = "Rollbar"
    RUNSCOPE = "Runscope"
    RUXIT = "Ruxit"
    SALESFORCESERVICECLOUD = "SalesForceServiceCloud"
    SALTSTACK = "SaltStack"
    SCALYR = "Scalyr"
    SCOM = "SCOM"
    SCOUT = "Scout"
    SEMATEXTSPM = "SematextSpm"
    SENSU = "Sensu"
    SENTRY = "Sentry"
    SERVERDENSITY = "ServerDensity"
    SERVERGUARD24 = "ServerGuard24"
    SERVICENOW = "ServiceNow"
    SERVICENOWV2 = "ServiceNowV2"
    SERVICENOWV3 = "ServiceNowV3"
    SIGNALFX = "Signalfx"
    SIGNALFXV2 = "SignalFXV2"
    SIGNALSCIENCES = "SignalSciences"
    SITE24X7 = "Site24x7"
    SLACK = "Slack"
    SLACKAPP = "SlackApp"
    SOASTA = "Soasta"
    SOLARWINDS = "Solarwinds"
    SOLARWINDSMSPNCENTRAL = "SolarwindsMSPNCentral"
    SOLARWINDSWEBHELPDESK = "SolarWindsWebHelpDesk"
    SPLUNK = "Splunk"
    SPLUNKITSI = "SplunkITSI"
    STACKDRIVER = "Stackdriver"
    STACKSTORM = "StackStorm"
    STATUSCAKE = "StatusCake"
    STATUSHUB = "StatusHub"
    STATUSIO = "StatusIO"
    STATUSPAGEIO = "StatusPageIO"
    STATUSY = "Statusy"
    STRUXUREWARE = "StruxureWare"
    SUMOLOGIC = "SumoLogic"
    SYSDIGCLOUD = "SysdigCloud"
    THOUSANDEYES = "ThousandEyes"
    THREATSTACK = "ThreatStack"
    THUNDRA = "Thundra"
    TIDEWAYS = "Tideways"
    TRACE = "Trace"
    TRACKIT = "TrackIt"
    TRAVISCI = "TravisCI"
    TWILIO = "Twilio"
    UPDOWNIO = "UpdownIO"
    UPTIMEROBOT = "UptimeRobot"
    UPTIMEROBOTEMAIL = "UptimeRobotEmail"
    UPTIMEWEBHOOK = "UptimeWebhook"
    UPTRENDSEMAIL = "UptrendsEmail"
    VCENTER = "VCenter"
    VCSA = "VCSA"
    VIVIDCORTEX = "VividCortex"
    WAVEFRONT = "Wavefront"
    WEBHOOK = "Webhook"
    WHATSUPGOLD = "WhatsUpGold"
    WORKATO = "Workato"
    XLRELEASE = "XLRelease"
    XMPP = "Xmpp"
    ZABBIX = "Zabbix"
    ZAPIER = "Zapier"
    ZENDESK = "Zendesk"
    ZENOSS = "Zenoss"
    ZYRIONEMAIL = "ZyrionEmail"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AlertFilterConditionMatchType(str, Enum):
    MATCH_ALL = "match-all"
    MATCH_ALL_CONDITIONS = "match-all-conditions"
    MATCH_ANY_CONDITION = "match-any-condition"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.timeline_period import TimelinePeriod


T = TypeVar("T", bound="TimelineRotation")


@_attrs_define
class TimelineRotation:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        order (Union[Unset, float]):
        periods (Union[Unset, List['TimelinePeriod']]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    order: Union[Unset, float] = UNSET
    periods: Union[Unset, List["TimelinePeriod"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        order = self.order

        periods: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.periods, Unset):
            periods = []
            for periods_item_data in self.periods:
                periods_item = periods_item_data.to_dict()
                periods.append(periods_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if order is not UNSET:
            field_dict["order"] = order
        if periods is not UNSET:
            field_dict["periods"] = periods

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.timeline_period import TimelinePeriod

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        order = d.pop("order", UNSET)

        periods = []
        _periods = d.pop("periods", UNSET)
        for periods_item_data in _periods or []:
            periods_item = TimelinePeriod.from_dict(periods_item_data)

            periods.append(periods_item)

        timeline_rotation = cls(
            id=id,
            name=name,
            order=order,
            periods=periods,
        )

        timeline_rotation.additional_properties = d
        return timeline_rotation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.bidirectional_callback_new_bidirectional_callback_type import (
    BidirectionalCallbackNewBidirectionalCallbackType,
)
from ..models.outgoing_callback_new_callback_type import OutgoingCallbackNewCallbackType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_mapping import ActionMapping
    from ..models.alert_filter import AlertFilter


T = TypeVar("T", bound="ServiceNowV3Callback")


@_attrs_define
class ServiceNowV3Callback:
    """
    Attributes:
        alert_filter (Union[Unset, AlertFilter]):
        forwarding_enabled (Union[Unset, bool]):
        forwarding_action_mappings (Union[Unset, List['ActionMapping']]):
        callback_type (Union[Unset, OutgoingCallbackNewCallbackType]):
        updates_action_mappings (Union[Unset, List['ActionMapping']]):
        updates_enabled (Union[Unset, bool]):
        bidirectional_callback_type (Union[Unset, BidirectionalCallbackNewBidirectionalCallbackType]):
        instance_subdomain (Union[Unset, str]):
        user_id (Union[Unset, str]):
        password (Union[Unset, str]):
    """

    alert_filter: Union[Unset, "AlertFilter"] = UNSET
    forwarding_enabled: Union[Unset, bool] = UNSET
    forwarding_action_mappings: Union[Unset, List["ActionMapping"]] = UNSET
    callback_type: Union[Unset, OutgoingCallbackNewCallbackType] = UNSET
    updates_action_mappings: Union[Unset, List["ActionMapping"]] = UNSET
    updates_enabled: Union[Unset, bool] = UNSET
    bidirectional_callback_type: Union[Unset, BidirectionalCallbackNewBidirectionalCallbackType] = UNSET
    instance_subdomain: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        alert_filter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.alert_filter, Unset):
            alert_filter = self.alert_filter.to_dict()

        forwarding_enabled = self.forwarding_enabled

        forwarding_action_mappings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.forwarding_action_mappings, Unset):
            forwarding_action_mappings = []
            for forwarding_action_mappings_item_data in self.forwarding_action_mappings:
                forwarding_action_mappings_item = forwarding_action_mappings_item_data.to_dict()
                forwarding_action_mappings.append(forwarding_action_mappings_item)

        callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.callback_type, Unset):
            callback_type = self.callback_type.value

        updates_action_mappings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.updates_action_mappings, Unset):
            updates_action_mappings = []
            for updates_action_mappings_item_data in self.updates_action_mappings:
                updates_action_mappings_item = updates_action_mappings_item_data.to_dict()
                updates_action_mappings.append(updates_action_mappings_item)

        updates_enabled = self.updates_enabled

        bidirectional_callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.bidirectional_callback_type, Unset):
            bidirectional_callback_type = self.bidirectional_callback_type.value

        instance_subdomain = self.instance_subdomain

        user_id = self.user_id

        password = self.password

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alert_filter is not UNSET:
            field_dict["alertFilter"] = alert_filter
        if forwarding_enabled is not UNSET:
            field_dict["forwardingEnabled"] = forwarding_enabled
        if forwarding_action_mappings is not UNSET:
            field_dict["forwardingActionMappings"] = forwarding_action_mappings
        if callback_type is not UNSET:
            field_dict["callback-type"] = callback_type
        if updates_action_mappings is not UNSET:
            field_dict["updatesActionMappings"] = updates_action_mappings
        if updates_enabled is not UNSET:
            field_dict["updatesEnabled"] = updates_enabled
        if bidirectional_callback_type is not UNSET:
            field_dict["bidirectional-callback-type"] = bidirectional_callback_type
        if instance_subdomain is not UNSET:
            field_dict["instanceSubdomain"] = instance_subdomain
        if user_id is not UNSET:
            field_dict["userId"] = user_id
        if password is not UNSET:
            field_dict["password"] = password

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.action_mapping import ActionMapping
        from ..models.alert_filter import AlertFilter

        d = src_dict.copy()
        _alert_filter = d.pop("alertFilter", UNSET)
        alert_filter: Union[Unset, AlertFilter]
        if isinstance(_alert_filter, Unset):
            alert_filter = UNSET
        else:
            alert_filter = AlertFilter.from_dict(_alert_filter)

        forwarding_enabled = d.pop("forwardingEnabled", UNSET)

        forwarding_action_mappings = []
        _forwarding_action_mappings = d.pop("forwardingActionMappings", UNSET)
        for forwarding_action_mappings_item_data in _forwarding_action_mappings or []:
            forwarding_action_mappings_item = ActionMapping.from_dict(forwarding_action_mappings_item_data)

            forwarding_action_mappings.append(forwarding_action_mappings_item)

        _callback_type = d.pop("callback-type", UNSET)
        callback_type: Union[Unset, OutgoingCallbackNewCallbackType]
        if isinstance(_callback_type, Unset):
            callback_type = UNSET
        else:
            callback_type = OutgoingCallbackNewCallbackType(_callback_type)

        updates_action_mappings = []
        _updates_action_mappings = d.pop("updatesActionMappings", UNSET)
        for updates_action_mappings_item_data in _updates_action_mappings or []:
            updates_action_mappings_item = ActionMapping.from_dict(updates_action_mappings_item_data)

            updates_action_mappings.append(updates_action_mappings_item)

        updates_enabled = d.pop("updatesEnabled", UNSET)

        _bidirectional_callback_type = d.pop("bidirectional-callback-type", UNSET)
        bidirectional_callback_type: Union[Unset, BidirectionalCallbackNewBidirectionalCallbackType]
        if isinstance(_bidirectional_callback_type, Unset):
            bidirectional_callback_type = UNSET
        else:
            bidirectional_callback_type = BidirectionalCallbackNewBidirectionalCallbackType(
                _bidirectional_callback_type
            )

        instance_subdomain = d.pop("instanceSubdomain", UNSET)

        user_id = d.pop("userId", UNSET)

        password = d.pop("password", UNSET)

        service_now_v3_callback = cls(
            alert_filter=alert_filter,
            forwarding_enabled=forwarding_enabled,
            forwarding_action_mappings=forwarding_action_mappings,
            callback_type=callback_type,
            updates_action_mappings=updates_action_mappings,
            updates_enabled=updates_enabled,
            bidirectional_callback_type=bidirectional_callback_type,
            instance_subdomain=instance_subdomain,
            user_id=user_id,
            password=password,
        )

        service_now_v3_callback.additional_properties = d
        return service_now_v3_callback

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OutgoingCallbackCallbackType(str, Enum):
    BIDIRECTIONAL_CALLBACK = "bidirectional-callback"
    CAMPFIRE_CALLBACK = "campfire-callback"
    FLOWDOCK_CALLBACK = "flowdock-callback"
    FLOWDOCK_V2_CALLBACK = "flowdock-v2-callback"
    PLANIO_CALLBACK = "planio-callback"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_schedule_rotation_payload import CreateScheduleRotationPayload
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="CreateSchedulePayload")


@_attrs_define
class CreateSchedulePayload:
    """
    Attributes:
        name (str): Name of the schedule
        description (Union[Unset, str]): The description of schedule
        timezone (Union[Unset, str]): Timezone of schedule
        enabled (Union[Unset, bool]): Enable/disable state of schedule
        owner_team (Union[Unset, TeamMeta]):
        rotations (Union[Unset, List['CreateScheduleRotationPayload']]):
    """

    name: str
    description: Union[Unset, str] = UNSET
    timezone: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    rotations: Union[Unset, List["CreateScheduleRotationPayload"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        timezone = self.timezone

        enabled = self.enabled

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        rotations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rotations, Unset):
            rotations = []
            for rotations_item_data in self.rotations:
                rotations_item = rotations_item_data.to_dict()
                rotations.append(rotations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if rotations is not UNSET:
            field_dict["rotations"] = rotations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_schedule_rotation_payload import CreateScheduleRotationPayload
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description", UNSET)

        timezone = d.pop("timezone", UNSET)

        enabled = d.pop("enabled", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        rotations = []
        _rotations = d.pop("rotations", UNSET)
        for rotations_item_data in _rotations or []:
            rotations_item = CreateScheduleRotationPayload.from_dict(rotations_item_data)

            rotations.append(rotations_item)

        create_schedule_payload = cls(
            name=name,
            description=description,
            timezone=timezone,
            enabled=enabled,
            owner_team=owner_team,
            rotations=rotations,
        )

        create_schedule_payload.additional_properties = d
        return create_schedule_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.time_restriction_interval_type import TimeRestrictionIntervalType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.weekday_time_restriction import WeekdayTimeRestriction


T = TypeVar("T", bound="WeekdayTimeRestrictionInterval")


@_attrs_define
class WeekdayTimeRestrictionInterval:
    """Weekday time restriction interval

    Attributes:
        type (TimeRestrictionIntervalType):
        restrictions (Union[Unset, List['WeekdayTimeRestriction']]):
    """

    type: TimeRestrictionIntervalType
    restrictions: Union[Unset, List["WeekdayTimeRestriction"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        restrictions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.restrictions, Unset):
            restrictions = []
            for restrictions_item_data in self.restrictions:
                restrictions_item = restrictions_item_data.to_dict()
                restrictions.append(restrictions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
            }
        )
        if restrictions is not UNSET:
            field_dict["restrictions"] = restrictions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.weekday_time_restriction import WeekdayTimeRestriction

        d = src_dict.copy()
        type = TimeRestrictionIntervalType(d.pop("type"))

        restrictions = []
        _restrictions = d.pop("restrictions", UNSET)
        for restrictions_item_data in _restrictions or []:
            restrictions_item = WeekdayTimeRestriction.from_dict(restrictions_item_data)

            restrictions.append(restrictions_item)

        weekday_time_restriction_interval = cls(
            type=type,
            restrictions=restrictions,
        )

        weekday_time_restriction_interval.additional_properties = d
        return weekday_time_restriction_interval

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.deduplication_action_deduplication_action_type import DeduplicationActionDeduplicationActionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.duration import Duration


T = TypeVar("T", bound="DeduplicationAction")


@_attrs_define
class DeduplicationAction:
    """
    Attributes:
        deduplication_action_type (DeduplicationActionDeduplicationActionType):
        count (int):
        duration (Union[Unset, Duration]):
    """

    deduplication_action_type: DeduplicationActionDeduplicationActionType
    count: int
    duration: Union[Unset, "Duration"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        deduplication_action_type = self.deduplication_action_type.value

        count = self.count

        duration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duration, Unset):
            duration = self.duration.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "deduplicationActionType": deduplication_action_type,
                "count": count,
            }
        )
        if duration is not UNSET:
            field_dict["duration"] = duration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.duration import Duration

        d = src_dict.copy()
        deduplication_action_type = DeduplicationActionDeduplicationActionType(d.pop("deduplicationActionType"))

        count = d.pop("count")

        _duration = d.pop("duration", UNSET)
        duration: Union[Unset, Duration]
        if isinstance(_duration, Unset):
            duration = UNSET
        else:
            duration = Duration.from_dict(_duration)

        deduplication_action = cls(
            deduplication_action_type=deduplication_action_type,
            count=count,
            duration=duration,
        )

        deduplication_action.additional_properties = d
        return deduplication_action

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.bidirectional_callback_bidirectional_callback_type import BidirectionalCallbackBidirectionalCallbackType
from ..models.outgoing_callback_callback_type import OutgoingCallbackCallbackType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_filter import AlertFilter


T = TypeVar("T", bound="HipChatCallbackV2")


@_attrs_define
class HipChatCallbackV2:
    """
    Attributes:
        alert_filter (Union[Unset, AlertFilter]):
        alert_actions (Union[Unset, List[str]]):
        callback_type (Union[Unset, OutgoingCallbackCallbackType]):
        send_alert_actions (Union[Unset, bool]):
        bidirectional_callback_type (Union[Unset, BidirectionalCallbackBidirectionalCallbackType]):
        rooms (Union[Unset, List[str]]):
        token (Union[Unset, str]):
        notify (Union[Unset, bool]):
        host_url (Union[Unset, str]):
    """

    alert_filter: Union[Unset, "AlertFilter"] = UNSET
    alert_actions: Union[Unset, List[str]] = UNSET
    callback_type: Union[Unset, OutgoingCallbackCallbackType] = UNSET
    send_alert_actions: Union[Unset, bool] = UNSET
    bidirectional_callback_type: Union[Unset, BidirectionalCallbackBidirectionalCallbackType] = UNSET
    rooms: Union[Unset, List[str]] = UNSET
    token: Union[Unset, str] = UNSET
    notify: Union[Unset, bool] = UNSET
    host_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        alert_filter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.alert_filter, Unset):
            alert_filter = self.alert_filter.to_dict()

        alert_actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.alert_actions, Unset):
            alert_actions = self.alert_actions

        callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.callback_type, Unset):
            callback_type = self.callback_type.value

        send_alert_actions = self.send_alert_actions

        bidirectional_callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.bidirectional_callback_type, Unset):
            bidirectional_callback_type = self.bidirectional_callback_type.value

        rooms: Union[Unset, List[str]] = UNSET
        if not isinstance(self.rooms, Unset):
            rooms = self.rooms

        token = self.token

        notify = self.notify

        host_url = self.host_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alert_filter is not UNSET:
            field_dict["alertFilter"] = alert_filter
        if alert_actions is not UNSET:
            field_dict["alertActions"] = alert_actions
        if callback_type is not UNSET:
            field_dict["callback-type"] = callback_type
        if send_alert_actions is not UNSET:
            field_dict["sendAlertActions"] = send_alert_actions
        if bidirectional_callback_type is not UNSET:
            field_dict["bidirectional-callback-type"] = bidirectional_callback_type
        if rooms is not UNSET:
            field_dict["rooms"] = rooms
        if token is not UNSET:
            field_dict["token"] = token
        if notify is not UNSET:
            field_dict["notify"] = notify
        if host_url is not UNSET:
            field_dict["hostUrl"] = host_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.alert_filter import AlertFilter

        d = src_dict.copy()
        _alert_filter = d.pop("alertFilter", UNSET)
        alert_filter: Union[Unset, AlertFilter]
        if isinstance(_alert_filter, Unset):
            alert_filter = UNSET
        else:
            alert_filter = AlertFilter.from_dict(_alert_filter)

        alert_actions = cast(List[str], d.pop("alertActions", UNSET))

        _callback_type = d.pop("callback-type", UNSET)
        callback_type: Union[Unset, OutgoingCallbackCallbackType]
        if isinstance(_callback_type, Unset):
            callback_type = UNSET
        else:
            callback_type = OutgoingCallbackCallbackType(_callback_type)

        send_alert_actions = d.pop("sendAlertActions", UNSET)

        _bidirectional_callback_type = d.pop("bidirectional-callback-type", UNSET)
        bidirectional_callback_type: Union[Unset, BidirectionalCallbackBidirectionalCallbackType]
        if isinstance(_bidirectional_callback_type, Unset):
            bidirectional_callback_type = UNSET
        else:
            bidirectional_callback_type = BidirectionalCallbackBidirectionalCallbackType(_bidirectional_callback_type)

        rooms = cast(List[str], d.pop("rooms", UNSET))

        token = d.pop("token", UNSET)

        notify = d.pop("notify", UNSET)

        host_url = d.pop("hostUrl", UNSET)

        hip_chat_callback_v2 = cls(
            alert_filter=alert_filter,
            alert_actions=alert_actions,
            callback_type=callback_type,
            send_alert_actions=send_alert_actions,
            bidirectional_callback_type=bidirectional_callback_type,
            rooms=rooms,
            token=token,
            notify=notify,
            host_url=host_url,
        )

        hip_chat_callback_v2.additional_properties = d
        return hip_chat_callback_v2

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TeamLogDetails")


@_attrs_define
class TeamLogDetails:
    """
    Attributes:
        owner (Union[Unset, str]):
        created_date (Union[Unset, datetime.datetime]):
        log (Union[Unset, str]):
    """

    owner: Union[Unset, str] = UNSET
    created_date: Union[Unset, datetime.datetime] = UNSET
    log: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        owner = self.owner

        created_date: Union[Unset, str] = UNSET
        if not isinstance(self.created_date, Unset):
            created_date = self.created_date.isoformat()

        log = self.log

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if owner is not UNSET:
            field_dict["owner"] = owner
        if created_date is not UNSET:
            field_dict["createdDate"] = created_date
        if log is not UNSET:
            field_dict["log"] = log

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        owner = d.pop("owner", UNSET)

        _created_date = d.pop("createdDate", UNSET)
        created_date: Union[Unset, datetime.datetime]
        if isinstance(_created_date, Unset):
            created_date = UNSET
        else:
            created_date = isoparse(_created_date)

        log = d.pop("log", UNSET)

        team_log_details = cls(
            owner=owner,
            created_date=created_date,
            log=log,
        )

        team_log_details.additional_properties = d
        return team_log_details

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
